unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, StdCtrls, Buttons;

type
  TForm1 = class(TForm)
    DrawGrid1: TDrawGrid;
    BitBtn1: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure DrawGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure AssignBrickColors;
    procedure RedrawCells;
    procedure RemoveBricks (cl : TColor; ACol, ARow : integer);
    function IsBrickAlone(ACol, ARow : integer) : Boolean;
  end;



   const
  MAX_COLORS = 3;
const
  PossibleColors : array [0..MAX_COLORS-1] of TColor = (clRed, clBlue, clGreen);
  const
  MAX_COLUMNS = 20;
  MAX_ROWS = 20;

  const
  WALL_COLOR : TColor = clWhite;


var
  Form1: TForm1;
   ColorOfBrick : array [0..MAX_COLUMNs, 0..MAX_ROWS] of TColor;



implementation

{$R *.dfm}

{ TForm1 }

procedure TForm1.AssignBrickColors;
var
  i, j : integer;
begin
 for i := 0 to Form1.DrawGrid1.ColCount-1 do
    for j := 0 to Form1.DrawGrid1.RowCount-1 do
      ColorOfBrick[i][j] := PossibleColors[Random(MAX_COLORS)]
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 Randomize;
 Bitbtn1.Click ;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
 AssignBrickColors;
  RedrawCells;
end;

procedure TForm1.DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
DrawGrid1.Canvas.Brush.Color := ColorOfBrick[ACol][ARow];
DrawGrid1.Canvas.FillRect(Rect)
end;

procedure TForm1.RedrawCells;
var
  i, j : integer;
begin
  for i := 0 to Form1.DrawGrid1.ColCount-1 do
    for j := 0 to Form1.DrawGrid1.RowCount-1 do
      Form1.DrawGrid1DrawCell(Form1, i, j, Form1.DrawGrid1.CellRect(i,j), [])
end;

procedure TForm1.RemoveBricks(cl: TColor; ACol, ARow: integer);
begin
// If the given color does not match with the color of the current brick,
  // Exit without doing anything

  if ColorOfBrick[ACol, ARow] <> cl then
    exit;

  // Remove the brick, by making its color
  // with the same as the color of the wall
  
ColorOfBrick[ACol, ARow] := WALL_COLOR;

  // Now, apply this to its neighbours recursively
  // But don't forget that this cell might be a border cell,
  // so that it may not have one of the left, right, up or down neighbours

  // left neighbour

  if ACol > 0 then
    RemoveBricks(cl, ACol-1, ARow);

  // right neighbour

  if ACol < Form1.DrawGrid1.ColCount-1 then
    RemoveBricks(cl, ACol+1, ARow);

  // upper neighbour

  if ARow > 0 then
    RemoveBricks(cl, ACol, ARow-1);

  // down neighbour

  if ARow < Form1.DrawGrid1.RowCount-1 then
    RemoveBricks(cl, ACol, ARow+1);
end;

function TForm1.IsBrickAlone(ACol, ARow: integer): Boolean;
begin
// Initially assume that it is alone.

  Result := True;

  // If there is not a brick on the clicked cell,
  // exit without doing nothing. This means that
  // an empty cell on the wall will be interpreted to be alone.

  if ColorOfBrick[ACol, ARow] = WALL_COLOR then
    exit;

  // Now, check all of its neighbours whether at least one of them
  // has the same color. Don't forget to check whether the cell
  // is a border cell or not. If it is a border cell, we can get
  // a memory violation error when trying to check one of its non-existent
  // neighbours.

  // Check the left neighbour

  if ACol > 0 then
    if ColorOfBrick[ACol-1, ARow] = ColorOfBrick[ACol, ARow] then
      Result := False;

  // Check the right neighbour

  if ACol < Form1.DrawGrid1.ColCount-1 then
    if ColorOfBrick[ACol+1, ARow] = ColorOfBrick[ACol, ARow] then
      Result := False;

  // Check the upper neighbour

  if ARow > 0 then
    if ColorOfBrick[ACol, ARow-1] = ColorOfBrick[ACol, ARow] then
      Result := False;

  // Check the down neighbour

  if ARow < Form1.DrawGrid1.RowCount-1 then
    if ColorOfBrick[ACol, ARow+1] = ColorOfBrick[ACol, ARow] then
      Result := False
end;

procedure TForm1.DrawGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  // Check if the cell is alone or not. If it is, we will do nothing
  // according to the game rule, but it is not alone, we should remove
  // that area from the wall. We do this by calling our recursive procedure
  // RemoveBricks.

  // If it is not alone, remove the block

  if not IsBrickAlone(ACol, ARow) then
  begin
    RemoveBricks(ColorOfBrick[ACol, ARow], ACol, ARow);

    // Redraw the cells
    RedrawCells
  end
end;

end.
